@@ -16,12 +16,42 @@
 #include <png.h>
 #include "gqcam.h"
 #include "frontend.h"
+#include "preferences.h"
 #include "save.h"
 #include "color.h"
 #include "greyscale.h"
+#include "timer.h"
+
+
+void closedialog(GtkWidget *widget, GtkWidget *dialog)
+{
+  gtk_widget_destroy(dialog);
+}
+
+
+void errordialog(char *message, struct Camera *camera)
+{
+  GtkWidget *errordialog;
+  GtkWidget *button;
+  GtkWidget *label;
+  
+  errordialog = gtk_dialog_new();
+
+  button = gtk_button_new_with_label("Oops...");
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(errordialog)->action_area), button, TRUE, TRUE, 0);
+  gtk_signal_connect (GTK_OBJECT(button), "clicked", (GtkSignalFunc) closedialog, errordialog);
+    
+  gtk_widget_show (button);
+
+  label = gtk_label_new(message);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(errordialog)->vbox), label, TRUE, TRUE, 8);
+  gtk_widget_show (label);
+  
+  gtk_widget_show(errordialog);
+
+  return;
+}
 
-GtkWidget *drawing_area;
-static GdkPixmap *pixmap = NULL;
 
 void setbrightness(GtkAdjustment *adj, struct Camera *camera)
 {
@@ -151,15 +181,14 @@ void en_dis_able(GtkWidget *disable, GtkWidget *enable)
     gtk_widget_set_sensitive(enable, TRUE);
   }
 }
-/*
+
 void camera_info(GtkWidget *widget, struct Camera *camera)
 {
-  print_cam_info(camera);
-}
-*/
-void camera_info(GtkWidget *widget, struct Camera *camera)
-{
-  GtkWidget *dialog, *textbox, *button, *hbox, *vscrollbar;
+  GtkWidget *dialog;
+  GtkWidget *textbox;
+  GtkWidget *button;
+  GtkWidget *hbox;
+  GtkWidget *vscrollbar;
   char buff[80];
   
   dialog = gtk_dialog_new();
@@ -309,6 +338,14 @@ void about_dialog(GtkWidget *widget, struct Camera *camera)
   gtk_widget_show_all (dialog);
 }
 
+/*
+void pref_dialog(GtkWidget *widget, struct Camera *camera)
+{
+  errordialog("Preferences dialog is not implemented yet...", camera);
+
+}
+*/
+
 void open_dialog_ok(GtkWidget *widget, struct Camera *camera)
 {
   stop_capture( NULL, camera );
@@ -493,6 +530,7 @@ int create_frontend(struct Camera *camera)
   GtkWidget *saveitem;
   GtkWidget *openitem;
   GtkWidget *closeitem;
+  GtkWidget *prefitem;
   GtkWidget *seperator;
   GtkWidget *cameramenu;
   GtkWidget *cameraitem;
@@ -500,6 +538,7 @@ int create_frontend(struct Camera *camera)
   GtkWidget *updateitem;
   GtkWidget *startitem;
   GtkWidget *stopitem;
+  GtkWidget *timeritem;
   GtkWidget *helpmenu;
   GtkWidget *helpitem;
   GtkWidget *aboutitem;
@@ -530,7 +569,7 @@ int create_frontend(struct Camera *camera)
   gtk_window_set_title (GTK_WINDOW (window), title);
   gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, TRUE);
   gtk_signal_connect (GTK_OBJECT (window), "delete_event",
-		      GTK_SIGNAL_FUNC (delete_event), NULL);
+		      GTK_SIGNAL_FUNC (delete_event), camera);
 
   /* Create topbox */
   topbox = gtk_vbox_new (FALSE, 0);
@@ -563,11 +602,21 @@ int create_frontend(struct Camera *camera)
   gtk_menu_append(GTK_MENU(filemenu), seperator);
   gtk_widget_show(seperator);
 
+  prefitem = gtk_menu_item_new_with_label("Preferences...");
+  gtk_menu_append(GTK_MENU(filemenu), prefitem);
+  gtk_signal_connect(GTK_OBJECT (prefitem), "activate",
+		     GTK_SIGNAL_FUNC (pref_dialog), camera);
+  gtk_widget_show(prefitem);
+
+  seperator = gtk_menu_item_new();
+  gtk_menu_append(GTK_MENU(filemenu), seperator);
+  gtk_widget_show(seperator);
+
   quititem = gtk_menu_item_new_with_label("Quit");
   gtk_menu_append(GTK_MENU(filemenu), quititem);
-  gtk_signal_connect_object(GTK_OBJECT (quititem), "activate",
-			    GTK_SIGNAL_FUNC (delete_event),
-			    NULL);
+  gtk_signal_connect(GTK_OBJECT (quititem), "activate",
+		     GTK_SIGNAL_FUNC (delete_event),
+		     camera);
   gtk_widget_show(quititem);
 
   fileitem = gtk_menu_item_new_with_label("File");
@@ -608,6 +657,13 @@ int create_frontend(struct Camera *camera)
   else
     gtk_widget_set_sensitive(stopitem, FALSE);
 
+  timeritem = gtk_menu_item_new_with_label("Set Timer...");
+  gtk_menu_append(GTK_MENU(cameramenu), timeritem);
+  gtk_signal_connect(GTK_OBJECT (timeritem), "activate",
+		     GTK_SIGNAL_FUNC (set_timer),
+		     camera);
+  gtk_widget_show(timeritem);
+
   /*
   updateitem = gtk_menu_item_new_with_label("Update Controls");
   gtk_menu_append(GTK_MENU(cameramenu), updateitem);
@@ -706,7 +762,7 @@ int create_frontend(struct Camera *camera)
   gtk_widget_show(button);
 
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
-	 	      GTK_SIGNAL_FUNC (delete_event), NULL);
+	 	      GTK_SIGNAL_FUNC (delete_event), camera);
 
   /* create the frames per second status */
   if( 1 ) {
