@@ -17,17 +17,12 @@
 #include "gqcam.h"
 #include "frontend.h"
 #include "save.h"
+#include "time.h"
 
-struct Page_Ppm page_ppm;
-struct Page_Jpeg page_jpeg;
-struct Page_Gif page_gif;
-GtkWidget *textbox;
-
-
-void select_ok(GtkWidget *widget, GdkEvent *event, struct Camera *camera)
+void select_ok(GtkWidget *widget, struct Camera *camera)
 {
-  gtk_entry_set_text(GTK_ENTRY(textbox), gtk_file_selection_get_filename(GTK_FILE_SELECTION (widget)));
-  gtk_widget_destroy(GTK_WIDGET(widget));
+  gtk_entry_set_text(GTK_ENTRY(camera->save_struct.textbox), gtk_file_selection_get_filename(GTK_FILE_SELECTION (camera->save_struct.fileselect)));
+  gtk_widget_destroy(camera->save_struct.fileselect);
   return;
 }
 
@@ -39,15 +34,15 @@ void select_cancel(GtkWidget *widget, GdkEvent *event, gpointer data)
 
 void file_select(GtkWidget *widget, struct Camera *camera)
 {
-  GtkWidget *fileselect;
 
   //  camera->saving = 1;
-  fileselect = gtk_file_selection_new("Save Image");
-  gtk_widget_show(fileselect);
+  camera->save_struct.fileselect = gtk_file_selection_new("Save Image");
+  gtk_file_selection_set_filename (GTK_FILE_SELECTION(camera->save_struct.fileselect), camera->savefileclean);
+  gtk_widget_show(camera->save_struct.fileselect);
 
-  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fileselect)->cancel_button), "clicked", (GtkSignalFunc) select_cancel, GTK_OBJECT (fileselect));
+  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (camera->save_struct.fileselect)->cancel_button), "clicked", (GtkSignalFunc) select_cancel, GTK_OBJECT (camera->save_struct.fileselect));
 
-  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fileselect)->ok_button), "clicked", (GtkSignalFunc) select_ok, GTK_OBJECT (fileselect));
+  gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION (camera->save_struct.fileselect)->ok_button), "clicked", (GtkSignalFunc) select_ok, camera);
 
   return;
 }
@@ -68,24 +63,39 @@ void save_ok(GtkWidget *widget, struct Camera *camera)
 
   png_byte interlace;
 
-  if(!(outfile = fopen(gtk_entry_get_text(GTK_ENTRY(textbox)), "w"))){
-    printf("Need a file name...\n");
+  if(strlen(gtk_entry_get_text(GTK_ENTRY(camera->save_struct.textbox))) > 255){
+    errordialog("File name too long...\n");
     return;
   }
+
+  strcpy(camera->savefile, gtk_entry_get_text(GTK_ENTRY(camera->save_struct.textbox)));
+  strcpy(camera->savefileclean, gtk_entry_get_text(GTK_ENTRY(camera->save_struct.textbox)));
+  camera->save_struct.isinfo = 1;
+
+  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(camera->save_struct.timebutton)))
+    savefile_append_time(camera);
   
   switch (camera->savetype) {
 
   case PPM:
     if (GTK_TOGGLE_BUTTON(camera->page_ppm.radio_raw_ppm)->active){
-      ppm_save(camera, outfile, RAW);
+      camera->save_struct.format = RAW;
     }
     else if (GTK_TOGGLE_BUTTON(camera->page_ppm.radio_ascii_ppm)->active){
-      ppm_save(camera, outfile, ASCII);
+      camera->save_struct.format = ASCII;
     }
+    if (camera->saving)
+      ppm_save(camera);
+
     break; 
     
   case JPEG:
-    jpeg_save(camera, outfile, GTK_ADJUSTMENT(camera->page_jpeg.smooth_adj_jpeg)->value, GTK_ADJUSTMENT(camera->page_jpeg.quality_adj_jpeg)->value, (GTK_TOGGLE_BUTTON(camera->page_jpeg.optimize_jpeg)->active)?1:0);
+    camera->save_struct.quality = GTK_ADJUSTMENT(camera->page_jpeg.quality_adj_jpeg)->value;
+    camera->save_struct.smoothness = GTK_ADJUSTMENT(camera->page_jpeg.smooth_adj_jpeg)->value;
+    camera->save_struct.optimize = (GTK_TOGGLE_BUTTON(camera->page_jpeg.optimize_jpeg)->active)?1:0;
+    if (camera->saving)
+      jpeg_save(camera);
+
     break;
 
   case GIF:
@@ -95,18 +105,17 @@ void save_ok(GtkWidget *widget, struct Camera *camera)
 
   case PNG:
     if(GTK_TOGGLE_BUTTON(camera->page_png.interlace_png)->active)
-      interlace = PNG_INTERLACE_ADAM7;
+      camera->save_struct.interlace = PNG_INTERLACE_ADAM7;
     else
-      interlace = PNG_INTERLACE_NONE;
-    png_save(camera, outfile, interlace, GTK_ADJUSTMENT(camera->page_png.compression_adj_png)->value);
+      camera->save_struct.interlace = PNG_INTERLACE_NONE;
+
+    camera->save_struct.compression = GTK_ADJUSTMENT(camera->page_png.compression_adj_png)->value;
+    if (camera->saving)
+      png_save(camera);
     break;
     
   }
 
-  fclose(outfile);
-
-  camera->saving = 0;
-  
   gtk_widget_destroy(widget->parent->parent->parent);
   pthread_mutex_lock( &camera->pref_mutex );
   if( !camera->frozen )
@@ -116,19 +125,25 @@ void save_ok(GtkWidget *widget, struct Camera *camera)
   return;
 }
 
-void png_save(struct Camera *camera, FILE *outfile, png_byte interlace, int compression)
+void png_save(struct Camera *camera)
 {
   FILE *fp;
   png_structp png_ptr;
   png_infop info_ptr;
   png_uint_32 k, height, width;
   png_bytep row_pointers[camera->vid_win.height];
-  
-  /* open the file */
-  fp = outfile;
-  if (fp == NULL)
-    return;
 
+  /* open the file */
+  if(!strcmp(camera->savefile, ""))
+    fp = stdout;
+  else if(camera->savefile != NULL)
+    fp = fopen(camera->savefile, "w");
+
+  if (fp == NULL){
+    errordialog("Error - Could not open file");
+    return;
+  }
+  
   /* Create and initialize the png_struct with the desired error handler
    * functions.  If you want to use the default stderr and longjump method,
    * you can supply NULL for the last three parameters.  We also check that
@@ -139,6 +154,7 @@ void png_save(struct Camera *camera, FILE *outfile, png_byte interlace, int comp
   
   if (png_ptr == NULL)
     {
+      errordialog("Error - could not create PNG");
       fclose(fp);
       return;
     }
@@ -176,10 +192,10 @@ void png_save(struct Camera *camera, FILE *outfile, png_byte interlace, int comp
    * currently be PNG_COMPRESSION_TYPE_BASE and PNG_FILTER_TYPE_BASE. REQUIRED
    */
   if(camera->greyscale)
-    png_set_IHDR(png_ptr, info_ptr, camera->vid_win.width, camera->vid_win.height, 8, PNG_COLOR_TYPE_GRAY, interlace, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
+    png_set_IHDR(png_ptr, info_ptr, camera->vid_win.width, camera->vid_win.height, 8, PNG_COLOR_TYPE_GRAY, camera->save_struct.interlace, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
   else
-    png_set_IHDR(png_ptr, info_ptr, camera->vid_win.width, camera->vid_win.height, 8, PNG_COLOR_TYPE_RGB, interlace, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
-  png_set_compression_level(png_ptr, compression);
+    png_set_IHDR(png_ptr, info_ptr, camera->vid_win.width, camera->vid_win.height, 8, PNG_COLOR_TYPE_RGB, camera->save_struct.interlace, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
+  png_set_compression_level(png_ptr, camera->save_struct.compression);
   
   /* set the palette if there is one.  REQUIRED for indexed-color images */
   //palette = (png_colorp)png_malloc(png_ptr, 256 * sizeof (png_color));
@@ -306,19 +322,31 @@ void png_save(struct Camera *camera, FILE *outfile, png_byte interlace, int comp
   png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
   
   /* close the file */
-  //fclose(fp);
+  fclose(fp);
   
   /* that's it */
+  
   return;
 }
 
-void ppm_save(struct Camera *camera, FILE *outfile, int type)
+void ppm_save(struct Camera *camera)
 {
+  FILE *outfile;
   unsigned char buff[3];
   int i;
+
+  if(!strcmp(camera->savefile, ""))
+    outfile = stdout;
+  else if(camera->savefile != NULL)
+    outfile = fopen(camera->savefile, "w");
   
+  if (outfile == NULL){
+    errordialog("Could not open file");
+    return;
+  }
+ 
   /*  Raw  */
-  if (type == RAW){
+  if (camera->save_struct.format == RAW){
     fprintf (outfile, "P6\n%d %d\n%d\n", camera->vid_win.width, camera->vid_win.height, 255);
     if (camera->greyscale){
       for (i = 0; i < camera->vid_win.width*camera->vid_win.height; i++){
@@ -336,7 +364,7 @@ void ppm_save(struct Camera *camera, FILE *outfile, int type)
   }
 
   /*  Ascii  */
-  else if (type == ASCII){
+  else if (camera->save_struct.format == ASCII){
     fprintf (outfile, "P3\n%d %d\n%d\n", camera->vid_win.width, camera->vid_win.height, 255);
     if (camera->greyscale) {
       for (i = 0; i < camera->vid_win.width*camera->vid_win.height; i++){ 
@@ -348,11 +376,14 @@ void ppm_save(struct Camera *camera, FILE *outfile, int type)
 	fprintf (outfile, "%03d %03d %03d\n", camera->pic[i], camera->pic[i+1], camera->pic[i+2]);
       }
   }
+
+  fclose(outfile);
 }
 
 
-void jpeg_save(struct Camera *camera, FILE *outfile, int smooth, int quality, int optimize)
+void jpeg_save(struct Camera *camera)
 {
+  FILE *outfile;
   int ls=0, x, y;
   int i;
   int lines, pixels_per_line;
@@ -362,6 +393,16 @@ void jpeg_save(struct Camera *camera, FILE *outfile, int smooth, int quality, in
   JSAMPROW row_pointer[1];     // pointer to JSAMPLE row(s)
   int row_stride;              // physical row width in image buffer
 
+  if(!strcmp(camera->savefile, ""))
+    outfile = stdout;
+  else if(camera->savefile != NULL)
+    outfile = fopen(camera->savefile, "w");
+
+  if (outfile == NULL){
+    errordialog("Could not open file");
+    return;
+  }
+
   cinfo.err = jpeg_std_error(&jerr);
   jpeg_create_compress(&cinfo);
   
@@ -378,13 +419,13 @@ void jpeg_save(struct Camera *camera, FILE *outfile, int smooth, int quality, in
     cinfo.in_color_space = JCS_RGB;
   }
   
-  cinfo.smoothing_factor = smooth;
-  if (smooth) {
+  cinfo.smoothing_factor = camera->save_struct.smoothness;
+  if (camera->save_struct.smoothness) {
     cinfo.optimize_coding = TRUE;
   }
   
   jpeg_set_defaults(&cinfo);
-  jpeg_set_quality(&cinfo, quality, TRUE);
+  jpeg_set_quality(&cinfo, camera->save_struct.quality, TRUE);
   jpeg_start_compress(&cinfo, TRUE);
   row_stride = cinfo.input_components*camera->vid_win.width;
   while (cinfo.next_scanline < cinfo.image_height) {
@@ -430,6 +471,7 @@ void jpeg_save(struct Camera *camera, FILE *outfile, int smooth, int quality, in
   jpeg_destroy_compress(&cinfo);
 */
 
+  fclose(outfile);
   return;
 }
 void save_cancel(GtkWidget *widget, struct Camera *camera)
@@ -483,8 +525,7 @@ void save_dialog(GtkWidget *widget, struct Camera *camera)
   GtkWidget *ppmitem;
   GtkWidget *jpegitem;
   GtkWidget *pngitem;
-
-  int openpage = camera->savetype;
+  //int openpage = camera->savetype;
 
   pthread_mutex_lock( &camera->pref_mutex );
   if( !camera->frozen )
@@ -502,21 +543,27 @@ void save_dialog(GtkWidget *widget, struct Camera *camera)
   
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
-  label = gtk_label_new("File");
+  label = gtk_label_new("File:");
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
   gtk_widget_show(label);
-  textbox = gtk_entry_new();
-  gtk_box_pack_start(GTK_BOX(hbox), textbox, FALSE, FALSE, 2);
-  gtk_widget_show(textbox);
+  camera->save_struct.textbox = gtk_entry_new();
+  gtk_box_pack_start(GTK_BOX(hbox), camera->save_struct.textbox, FALSE, FALSE, 2);
+  if(camera->save_struct.isinfo)
+    gtk_entry_set_text(GTK_ENTRY(camera->save_struct.textbox), camera->savefileclean);
+  gtk_widget_show(camera->save_struct.textbox);
   button = gtk_button_new_with_label("Browse...");
-  gtk_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (file_select), NULL);
+  gtk_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (file_select), camera);
   gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 2);
   gtk_widget_show(button);
   gtk_widget_show(hbox);
 
-  typemenubutton = gtk_option_menu_new();
-  gtk_box_pack_start(GTK_BOX(vbox), typemenubutton, FALSE, FALSE, 2);
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
 
+  typemenubutton = gtk_option_menu_new();
+  gtk_box_pack_start(GTK_BOX(hbox), typemenubutton, FALSE, FALSE, 2);
+  gtk_widget_show(hbox);
+  
   typemenu = gtk_menu_new();
 
   pngitem = gtk_menu_item_new_with_label("png");
@@ -537,12 +584,15 @@ void save_dialog(GtkWidget *widget, struct Camera *camera)
                      GTK_SIGNAL_FUNC (switch_page_jpeg), camera);
   gtk_widget_show(jpegitem);
 
-
-
   gtk_option_menu_set_menu(GTK_OPTION_MENU(typemenubutton), typemenu);
+  gtk_option_menu_set_history(GTK_OPTION_MENU(typemenubutton), camera->savetype);
 
   gtk_widget_show(typemenubutton);
 
+  camera->save_struct.timebutton = gtk_check_button_new_with_label("Append time to filename");
+  gtk_box_pack_start(GTK_BOX(hbox), camera->save_struct.timebutton, FALSE, FALSE, 2);
+  gtk_widget_show(camera->save_struct.timebutton);
+  
 
   /* PPM PAGE */
 
@@ -561,9 +611,12 @@ void save_dialog(GtkWidget *widget, struct Camera *camera)
   camera->page_ppm.radio_ascii_ppm = gtk_radio_button_new_with_label(camera->page_ppm.group_ppm, "ASCII");
   gtk_box_pack_start(GTK_BOX(camera->page_ppm.vbox_ppm), camera->page_ppm.radio_ascii_ppm, FALSE, FALSE, 2);
   gtk_widget_show(camera->page_ppm.radio_ascii_ppm);
-  //gtk_widget_show(camera->page_ppm.frame_ppm);
-  //camera->currentsavepage = camera->page_ppm.frame_ppm;
 
+  if(camera->savetype == PPM){
+    gtk_widget_show(camera->page_ppm.frame_ppm);
+    camera->currentsavepage = camera->page_ppm.frame_ppm;
+  }
+  
   gtk_box_pack_start(GTK_BOX(vbox), camera->page_ppm.frame_ppm, FALSE, FALSE, 2);
 
 
@@ -606,10 +659,13 @@ void save_dialog(GtkWidget *widget, struct Camera *camera)
   camera->page_jpeg.optimize_jpeg = gtk_check_button_new_with_label("Optimize");
   gtk_table_attach_defaults(GTK_TABLE(camera->page_jpeg.table_jpeg), camera->page_jpeg.optimize_jpeg, 0, 1, 2, 3);
   gtk_widget_show(camera->page_jpeg.optimize_jpeg);
-  //  gtk_widget_show(camera->page_jpeg.frame_jpeg);
-
   gtk_box_pack_start(GTK_BOX(vbox), camera->page_jpeg.frame_jpeg, FALSE, FALSE, 2);
 
+  if(camera->savetype == JPEG){
+    gtk_widget_show(camera->page_jpeg.frame_jpeg);
+    camera->currentsavepage = camera->page_jpeg.frame_jpeg;
+  }
+
 /* PNG PAGE */
 
   camera->page_png.frame_png = gtk_frame_new(NULL);
@@ -643,9 +699,11 @@ void save_dialog(GtkWidget *widget, struct Camera *camera)
   
   gtk_box_pack_start(GTK_BOX(vbox), camera->page_png.frame_png, FALSE, FALSE, 2);
 
-  gtk_widget_show(camera->page_png.frame_png);
-  camera->currentsavepage = camera->page_png.frame_png;
-
+  if(camera->savetype == PNG){
+    gtk_widget_show(camera->page_png.frame_png);
+    camera->currentsavepage = camera->page_png.frame_png;
+  }
+  
 /* GIF PAGE */
 /*
   page_gif.frame_gif = gtk_frame_new(NULL);
@@ -694,6 +752,23 @@ void save_dialog(GtkWidget *widget, struct Camera *camera)
   return;
 }
 
+void savefile_append_time(struct Camera *camera)
+{
+  char *dotlocation;
+  char prefix[255+11], curtime[10];
+  int length;
 
-
-
+  dotlocation = strrchr(camera->savefileclean, '.');
+  if(dotlocation == NULL)
+    length = strlen(camera->savefileclean);
+  else
+    length = (dotlocation - camera->savefileclean)/(sizeof(char));
+  strncpy(prefix, camera->savefileclean, length);
+  sprintf(curtime, ".%d", time(NULL));
+  strcpy(prefix+length, curtime);
+  strcpy(prefix+length+strlen(curtime), camera->savefileclean+length);
+  //printf("%s\n", prefix);
+  
+  strcpy(camera->savefile, prefix);
+  
+}
