@@ -35,9 +35,9 @@ struct Page_Gif
 */
 
 enum Save_Types {
-  PPM = 0,
-  JPEG = 1,
-  PNG = 2,
+  PNG = 0,
+  PPM = 1,
+  JPEG = 2,
   GIF = 3
 };
 
@@ -48,6 +48,7 @@ enum Ppm_Types {
 
 void save_dialog(GtkWidget*, struct Camera*);
 void file_select(GtkWidget *widget, struct Camera *camera);
-void jpeg_save(struct Camera *camera, FILE *outfile, int smooth, int quality, int optimize);
-void ppm_save(struct Camera *camera, FILE *outfile, int type);
-void png_save(struct Camera *camera, FILE *outfile, png_byte interlace, int compression);
+void jpeg_save(struct Camera *camera);
+void ppm_save(struct Camera *camera);
+void png_save(struct Camera *camera);
+void savefile_append_time(struct Camera *camera);
